<?php

namespace FireDrum;

if ( !defined( 'ABSPATH' ) ) {
	die();
}

/**
 * Class Email_Marketing_Woocommerce
 */
class Email_Marketing_Woocommerce {
	private $plugin;

	private static $instance = null;

	public static function instantiate() {
		if ( self::$instance === null && self::is_woocommerce_installed() ) {
			self::$instance = new self();
		}
		return self::$instance;
	}

	public static function upgrade($current_db_version) {
		$wpdb = Email_Marketing_Plugin::instantiate()->get_wpdb();

		switch ( $current_db_version ) {
			case '0':
				require_once ( ABSPATH . 'wp-admin/includes/upgrade.php' );

				$collate = '';
				if ( $wpdb->has_cap( 'collation' ) ) {
					$collate = $wpdb->get_charset_collate();
				}

				$sql_query = "
				CREATE TABLE `{$wpdb->prefix}" . Email_Marketing_Plugin::get_branding() . "_wc_customer_carts` (
					`id` BIGINT(20) NOT NULL AUTO_INCREMENT,
					`user_id` BIGINT(20) NOT NULL DEFAULT 0,
					`first_name` VARCHAR(100) NOT NULL,
					`last_name` VARCHAR(100) NOT NULL,
					`user_email` VARCHAR(100) NOT NULL,
					`cart_empty` TINYINT(1) NOT NULL,
					`cart_items` LONGTEXT NOT NULL,
					`cart_total` DOUBLE(10, 2) NOT NULL,
					`date_updated` DATETIME NOT NULL,
					`sent_abandoned_cart_email` TINYINT(1) NOT NULL DEFAULT 0,
					PRIMARY KEY (`id`),
					KEY `user_id` (`user_id`),
					KEY `user_email` (`user_email`),
					KEY `cart_empty_sent_abandoned_cart_email_date_updated` (`cart_empty`, `sent_abandoned_cart_email`, `date_updated`)
				) {$collate};";

				dbDelta( $sql_query );

			case '1.46':
			case '1.47':
			case '1.48':
			case '1.49':
			case '1.50':
				// The next upgrade code goes here
			default:
				break;
		}
	}

	public static function uninstall($wpdb) {
		delete_option( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_enabled' );
		delete_option( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_placement' );
		delete_option( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_default' );
		delete_option( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_label' );
		delete_option( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_lists' );

		delete_option( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_enabled' );
		delete_option( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_threshold_value' );
		delete_option( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_threshold_unit' );
		delete_option( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_campaign' );

		wp_unschedule_event( wp_next_scheduled( Email_Marketing_Plugin::get_branding() . '_wc_send_abandoned_cart_emails_hook' ), Email_Marketing_Plugin::get_branding() . '_wc_send_abandoned_cart_emails_hook' );

		$wpdb->get_results( "DROP TABLE {$wpdb->prefix}" . Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_sent' );
		$wpdb->get_results( "DROP TABLE {$wpdb->prefix}" . Email_Marketing_Plugin::get_branding() . '_wc_customer_carts' );
	}

	/**
	 * Class constructor.
	 */
	private function __construct() {
		require_once( 'cart.class.php' );

		$this->plugin = Email_Marketing_Plugin::instantiate();
		$this->cart = Email_Marketing_Woocommerce_Cart::instantiate();

		add_action( 'admin_init', array( $this, 'on_admin_init' ) );
		add_action( 'init', array( $this, 'on_init' ) );
	}

	public function on_admin_init() {
		add_action( 'wp_ajax_' . Email_Marketing_Plugin::get_branding() . '_wc_admin_main_css', array (
			$this,
			'do_ajax_admin_main_css'
		) );
		add_action( 'wp_ajax_' . Email_Marketing_Plugin::get_branding() . '_wc_admin_main_js', array (
			$this,
			'do_ajax_admin_main_js'
		) );

		$this->register_settings();
	}

	public function on_init() {
		add_filter( Email_Marketing_Plugin::get_branding() . '_form_locations', array( $this, 'get_form_locations' ) );

		if ( is_admin() ) {
			$this->init_admin();
		} else {
			$this->init_frontend();
		}
	}

	public function init_admin() {
		wp_enqueue_style( Email_Marketing_Plugin::get_branding() . '_wc_admin_main', add_query_arg( array( 'action' => Email_Marketing_Plugin::get_branding() . '_wc_admin_main_css', 'ver' => Email_Marketing_Plugin::get_version() ), admin_url( 'admin-ajax.php' ) ) );

		wp_enqueue_script( Email_Marketing_Plugin::get_branding() . '_wc_admin_main', add_query_arg( array( 'action' => Email_Marketing_Plugin::get_branding() . '_wc_admin_main_js', 'ver' => Email_Marketing_Plugin::get_version() ), admin_url( 'admin-ajax.php' ) ), array (
			'jquery'
		), Email_Marketing_Plugin::get_version() );

		add_action( 'admin_menu', array (
			$this,
			'admin_menu'
		) );
	}

	public function init_frontend() {
		if ( get_option( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_enabled', '1' ) === '1' ) {
			add_action( get_option( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_placement' ), array (
				$this,
				'render_checkout_optin'
			) );
			add_action( 'woocommerce_checkout_order_processed', array( $this, 'maybe_subscribe_customer' ), 10, 2 );
		}
	}

	public function admin_menu() {
		add_submenu_page( Email_Marketing_Plugin::get_branding(), __( Email_Marketing_Plugin::get_menu_branding() . ' Settings for WooCommerce', Email_Marketing_Plugin::get_textdomain() ), 'WooCommerce', 'manage_options', Email_Marketing_Plugin::get_branding() . '-wc', array (
			$this,
			'render_woocommerce_page'
		) );
	}

	public function register_settings() {
		register_setting( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_settings', Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_enabled', array( $this, 'sanitize_checkout_optin_enabled' ) );
		register_setting( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_settings', Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_label', array( $this, 'sanitize_checkout_optin_label' ) );
		register_setting( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_settings', Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_default', array( $this, 'sanitize_checkout_optin_default' ) );
		register_setting( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_settings', Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_lists', array( $this, 'sanitize_checkout_optin_lists' ) );
		register_setting( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_settings', Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_placement', array( $this, 'sanitize_checkout_optin_placement' ) );

		register_setting( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_settings', Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_enabled', array( $this, 'sanitize_abandoned_cart_emails_enabled' ) );
		register_setting( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_settings', Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_threshold_value', array( $this, 'sanitize_abandoned_cart_emails_threshold_value' ) );
		register_setting( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_settings', Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_threshold_unit', array( $this, 'sanitize_abandoned_cart_emails_threshold_unit' ) );
		register_setting( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_settings', Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_campaign', array( $this, 'sanitize_abandoned_cart_emails_campaign' ) );

		// Checkout Opt-In Settings
		add_settings_section(
			Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_settings_section',
			null, // Section Title
			null, // Section description callback
			Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_settings'
		);

		add_settings_field(
			Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_enabled',
			__( 'Checkout Opt-in', Email_Marketing_Plugin::get_textdomain() ),
			array( $this, 'render_checkout_optin_enabled_setting_field' ),
			Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_settings',
			Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_settings_section'
		);
		add_settings_field(
			Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_label',
			__( 'Opt-in Checkbox Label', Email_Marketing_Plugin::get_textdomain() ),
			array( $this, 'render_checkout_optin_label_setting_field' ),
			Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_settings',
			Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_settings_section',
			array(
				'label_for' => Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_label',
				'class' => Email_Marketing_Plugin::get_branding() . '_wc_checkout_option'
			)
		);
		add_settings_field(
			Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_default',
			__( 'Opt-in Checkbox Default', Email_Marketing_Plugin::get_textdomain() ),
			array( $this, 'render_checkout_optin_default_setting_field' ),
			Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_settings',
			Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_settings_section',
			array(
				'class' => Email_Marketing_Plugin::get_branding() . '_wc_checkout_option'
			)
		);
		add_settings_field(
			Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_lists',
			__( 'Contact Lists', Email_Marketing_Plugin::get_textdomain() ),
			array( $this, 'render_checkout_optin_lists_setting_field' ),
			Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_settings',
			Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_settings_section',
			array(
				'class' => Email_Marketing_Plugin::get_branding() . '_wc_checkout_option'
			)
		);
		add_settings_field(
			Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_placement',
			__( 'Placement', Email_Marketing_Plugin::get_textdomain() ),
			array( $this, 'render_checkout_optin_placement_setting_field' ),
			Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_settings',
			Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_settings_section',
			array(
				'label_for' => Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_placement',
				'class' => Email_Marketing_Plugin::get_branding() . '_wc_checkout_option'
			)
		);

		// Abandoned Cart Emails Settings
		add_settings_section(
			Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_settings_section',
			null, // Section Title
			null, // Section description callback
			Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_settings'
		);

		add_settings_field(
			Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_enabled',
			__( 'Abandoned Cart Emails', Email_Marketing_Plugin::get_textdomain() ),
			array( $this, 'render_abandoned_cart_emails_enabled_setting_field' ),
			Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_settings',
			Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_settings_section'
		);
		add_settings_field(
			Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_threshold',
			__( 'Threshold', Email_Marketing_Plugin::get_textdomain() ),
			array( $this, 'render_abandoned_cart_emails_threshold_setting_field' ),
			Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_settings',
			Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_settings_section',
			array(
				'label_for' => Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_threshold_value',
				'class' => Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_option'
			)
		);
		add_settings_field(
			Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_campaign',
			__( 'Campaign', Email_Marketing_Plugin::get_textdomain() ),
			array( $this, 'render_abandoned_cart_emails_campaign_setting_field' ),
			Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_settings',
			Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_settings_section',
			array(
				'label_for' => Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_campaign',
				'class' => Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_option'
			)
		);
	}

	public function sanitize_checkout_optin_enabled($enabled) {
		switch ( $enabled ) {
			case '0':
			case '1':
				break;
			default:
				$enabled = get_option( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_enabled', '1' );
				break;
		}

		return $enabled;
	}

	public function sanitize_checkout_optin_label($label) {
		$label = sanitize_text_field( $label );

		if ( $label === '' ) {
			add_settings_error( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_label', 'label-missing', __( 'Checkout Opt-in Label cannot be blank.', Email_Marketing_Plugin::get_textdomain() ) );
			$label = get_option( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_label' );
		}

		return $label;
	}

	public function sanitize_checkout_optin_default($default) {
		switch ( $default ) {
			case '0':
			case '1':
				break;
			default:
				$default = get_option( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_default', '0' );
				break;
		}

		return $default;
	}

	public function sanitize_checkout_optin_lists($lists) {
		$categories = $this->plugin->get_api()->get_categories_from_api();
		if ( isset( $categories['categories'] ) && is_array( $categories['categories'] ) ) {
			if ( is_array( $lists ) ) {
				$lists = array_filter( $lists, 'is_numeric' );
			}

			if ( empty( $lists ) || !is_array( $lists ) ) {
				add_settings_error( Email_Marketing_Plugin::get_branding() . '_wc_checkbox_optin_lists', 'lists-not-selected', __( 'You must select at least one contact list.', Email_Marketing_Plugin::get_textdomain() ) );
				$lists = '';
			} else {
				$lists = implode( ',', $lists );
			}
		} else {
			$lists = '';
		}

		return $lists;
	}

	public function sanitize_checkout_optin_placement($placement) {
		$placement = sanitize_text_field( $placement );

		if ( $placement === '' || !in_array( $placement, array_keys( $this->get_checkout_locations() ) ) ) {
			add_settings_error( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_placement', 'placement-invalid', __( 'The selected Checkout Opt-in Placement is invalid.', Email_Marketing_Plugin::get_textdomain() ) );
			$placement = get_option( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_placement' );
		}

		return $placement;
	}

	public function sanitize_abandoned_cart_emails_enabled($enabled) {
		switch ( $enabled ) {
			case '0':
			case '1':
				break;
			default:
				$enabled = get_option( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_enabled', '1' );
				break;
		}

		return $enabled;
	}

	public function sanitize_abandoned_cart_emails_threshold_value($threshold_value) {
		$threshold_value = sanitize_text_field( $threshold_value );
		if ( !is_numeric( $threshold_value ) || intval( $threshold_value ) < 0 ) {
			add_settings_error( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_threshold_value', 'threshold-value-invalid', __( 'The Abandoned Cart Threshold must have a positive numeric value.', Email_Marketing_Plugin::get_textdomain() ) );
			$threshold_value = get_option( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_threshold_value', '1' );
		}
		return $threshold_value;
	}

	public function sanitize_abandoned_cart_emails_threshold_unit($threshold_unit) {
		switch ( $threshold_unit ) {
			case 'minutes':
			case 'hours':
			case 'days':
				break;
			default:
				$threshold_unit = get_option( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_threshold_unit', 'hours' );
				break;
		}

		return $threshold_unit;
	}

	public function sanitize_abandoned_cart_emails_campaign($campaign) {
		$campaigns = $this->plugin->get_api()->get_campaigns_from_api();
		if ( isset( $campaigns['campaigns'] ) && is_array( $campaigns['campaigns'] ) ) {
			if ( !is_numeric( $campaign ) ) {
				add_settings_error( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_campaign', 'campaign-invalid', __( 'You must select a valid Campaign.', Email_Marketing_Plugin::get_textdomain() ) );
				$campaign = get_option( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_campaign' );
			}
		} else {
			$campaign = '';
		}

		return $campaign;
	}

	public function render_checkout_optin_enabled_setting_field() {
		printf(
			'<fieldset>
				<legend class="screen-reader-text"><span>%2$s</span></legend>
				<input name="%1$s_wc_checkout_optin_enabled" type="hidden" value="0">
				<label>
					<input name="%1$s_wc_checkout_optin_enabled" type="checkbox" id="%1$s_wc_checkout_optin_enabled" value="1"%3$s>
					%4$s
				</label>
			</fieldset>',
			Email_Marketing_Plugin::get_branding(),
			esc_html__( 'Checkout Opt-in', Email_Marketing_Plugin::get_textdomain() ),
			get_option( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_enabled', '1' ) === '1' ? ' checked' : '',
			esc_html__( 'Enabled', Email_Marketing_Plugin::get_textdomain() )
		);
	}

	public function render_checkout_optin_label_setting_field() {
		printf(
			'<input name="%1$s_wc_checkout_optin_label" type="text" id="%1$s_wc_checkout_optin_label" value="%2$s" class="regular-text" required>
			<p class="description">%3$s</p>',
			Email_Marketing_Plugin::get_branding(),
			esc_attr( get_option( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_label', __( 'Subscribe to our Newsletter', Email_Marketing_Plugin::get_textdomain() ) ) ),
			esc_html__( 'Enter the opt-in checkbox label customers will see during checkout.', Email_Marketing_Plugin::get_textdomain() )
		);
	}

	public function render_checkout_optin_default_setting_field() {
		$current_value = get_option( Email_Marketing_Plugin::get_branding() . '_wc_checkbox_optin_default', '0' );
		printf(
			'<fieldset>
				<legend class="screen-reader-text"><span>%2$s</span></legend>
				<label><input type="radio" name="%1$s_wc_checkout_optin_default" value="0"%3$s> %4$s</label><br>
				<label><input type="radio" name="%1$s_wc_checkout_optin_default" value="1"%5$s> %6$s</label><br>
			</fieldset>',
			Email_Marketing_Plugin::get_branding(),
			esc_html__( 'Opt-in Checkbox Default', Email_Marketing_Plugin::get_textdomain() ),
			'0' === $current_value ? ' checked' : '',
			esc_html__( 'Unchecked — Recommended (Required for GDPR compliance)', Email_Marketing_Plugin::get_textdomain() ),
			'1' === $current_value ? ' checked' : '',
			esc_html__( 'Checked', Email_Marketing_Plugin::get_textdomain() )
		);
	}

	public function render_checkout_optin_lists_setting_field() {
		$current_values = explode( ',', get_option( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_lists' ) );
		$content = null;

		$categories = $this->plugin->get_api()->get_categories_from_api();
		if ( isset( $categories['categories'] ) && is_array( $categories['categories'] ) ) {
			$content = '';
			foreach ( $categories['categories'] as $category ) {
				$content .= sprintf(
					'<label>
						<input name="%1$s_wc_checkout_optin_lists[]" type="checkbox" value="%2$d"%3$s>
						%4$s
					</label>
					<br>',
					Email_Marketing_Plugin::get_branding(),
					$category['id'],
					in_array( $category['id'], $current_values ) ? ' checked' : '',
					$category['name']
				);
			}
		} else {
			$content = sprintf( '<p><strong>%1$s</strong></p>', esc_html__( 'No Lists Found', Email_Marketing_Plugin::get_textdomain() ) );
		}

		printf(
			'<fieldset>
				<legend class="screen-reader-text"><span>%1$s</span></legend>
				%2$s
				<p class="description">%3$s</p>
			</fieldset>',
			esc_html__( 'Contact Lists', Email_Marketing_Plugin::get_textdomain() ),
			$content,
			esc_html__( 'Contacts who opt-in will be assigned to these lists.', Email_Marketing_Plugin::get_textdomain() )
		);
	}

	public function render_checkout_optin_placement_setting_field() {
		$current_value = get_option( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_placement' );
		$options = '';

		$locations = $this->get_checkout_locations();
		foreach ( $locations as $value => $label ) {
			$options .= sprintf( '<option value="%1$s"%2$s>%3$s</option>', esc_attr( $value ), $value === $current_value ? ' selected' : '', esc_html( $label ) );
		}

		printf(
			'<select name="%1$s_wc_checkout_optin_placement" id="%1$s_wc_checkout_optin_placement">
				%2$s
			</select>
			<p class="description">%3$s</p>',
			Email_Marketing_Plugin::get_branding(),
			$options,
			esc_html__( 'Your checkout opt-in form will appear in this WooCommerce location.', Email_Marketing_Plugin::get_textdomain() )
		);
	}

	public function render_abandoned_cart_emails_enabled_setting_field() {
		printf(
			'<fieldset>
				<legend class="screen-reader-text"><span>%2$s</span></legend>
				<input name="%1$s_wc_abandoned_cart_emails_enabled" type="hidden" value="0">
				<label>
					<input name="%1$s_wc_abandoned_cart_emails_enabled" type="checkbox" id="%1$s_wc_abandoned_cart_emails_enabled" value="1"%3$s>
					%4$s
				</label>
			</fieldset>',
			Email_Marketing_Plugin::get_branding(),
			esc_html__( 'Abandoned Cart Emails', Email_Marketing_Plugin::get_textdomain() ),
			get_option( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_enabled', '1' ) === '1' ? ' checked' : '',
			esc_html__( 'Enabled', Email_Marketing_Plugin::get_textdomain() )
		);
	}

	public function render_abandoned_cart_emails_threshold_setting_field() {
		$current_value = get_option( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_threshold_value', '1' );
		$current_unit = get_option( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_threshold_unit', 'hours' );

		printf(
			'<input name="%1$s_wc_abandoned_cart_emails_threshold_value" type="number" step="1" min="1" id="%1$s_wc_abandoned_cart_emails_threshold_value" value="%2$s" class="small-text" required>
			<select name="%1$s_wc_abandoned_cart_emails_threshold_unit" id="%1$s_wc_abandoned_cart_emails_threshold_unit">
				<option value="minutes"%3$s>%4$s</option>
				<option value="hours"%5$s>%6$s</option>
				<option value="days"%7$s>%8$s</option>
			</select>
			<p class="description">%9$s</p>',
			Email_Marketing_Plugin::get_branding(),
			esc_attr( $current_value ),
			'minutes' === $current_unit ? ' selected' : '',
			esc_html__( 'Minutes', Email_Marketing_Plugin::get_textdomain() ),
			'hours' === $current_unit ? ' selected' : '',
			esc_html__( 'Hours', Email_Marketing_Plugin::get_textdomain() ),
			'days' === $current_unit ? ' selected' : '',
			esc_html__( 'Days', Email_Marketing_Plugin::get_textdomain() ),
			esc_html__( 'Carts will be considered abandoned if no customer activity is observed for at least this much time.', Email_Marketing_Plugin::get_textdomain() )
		);
	}

	public function render_abandoned_cart_emails_campaign_setting_field() {
		$current_value = intval( get_option( Email_Marketing_Plugin::get_branding() . '_wc_abandoned_cart_emails_campaign', '0' ) );
		$campaigns = $this->plugin->get_api()->get_campaigns_from_api();
		if ( isset( $campaigns['campaigns'] ) && is_array( $campaigns['campaigns'] ) ) {
			$options = '';
			foreach ( $campaigns['campaigns'] as $campaign ) {
				$options .= sprintf( '<option value="%1$d"%2$s>%3$s</option>', $campaign['id'], intval( $campaign['id'] ) === $current_value ? ' selected' : '', $campaign['name'] );
			}

			printf( '
				<select name="%1$s_wc_abandoned_cart_emails_campaign" id="%1$s_wc_abandoned_cart_emails_campaign">
					%2$s
				</select>
				<p class="description">%3$s</p>',
				Email_Marketing_Plugin::get_branding(),
				$options,
				esc_html__( 'The most recent issue of this campaign will be sent to customers once their cart is deemed abandoned.', Email_Marketing_Plugin::get_textdomain() )
			);
		} else {
			printf( '<p><strong>%1$s</strong></p>', esc_html__( 'No Campaigns Found', Email_Marketing_Plugin::get_textdomain() ) );
		}
	}

	public function render_woocommerce_page() {
		require( 'templates/woocommerce.php' );
	}

	public function do_ajax_admin_main_css() {
		header( 'Content-Type: text/css; charset=utf-8' );

		require( 'admin/css/main.css.php' );

		exit();
	}

	public function do_ajax_admin_main_js() {
		header( 'Content-Type: text/javascript; charset=utf-8' );

		require( 'admin/js/main.js.php' );

		exit();
	}

	public function render_checkout_optin($checkout) {
		$checked = $checkout->get_value( Email_Marketing_Plugin::get_branding() . '_checkout_optin' );
		if ( $checked === '' ) {
			$checked = get_option( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_default', '0' ) === '1';
		}
		
		woocommerce_form_field( Email_Marketing_Plugin::get_branding() . '_checkout_optin', array (
			'type' => 'checkbox',
			'class' => array (
				'form-row form-row-wide'
			),
			'label' => get_option( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_label', __( 'Subscribe to our Newsletter', Email_Marketing_Plugin::get_textdomain() ) )
		), $checked );
	}

	public function get_form_locations($locations) {
		$locations['woocommerce'] = __( 'WooCommerce Pages', Email_Marketing_Plugin::get_textdomain() );

		return $locations;
	}

	public function get_checkout_locations() {
		return apply_filters( Email_Marketing_Plugin::get_branding() . '_wc_checkout_locations', array(
			'woocommerce_before_checkout_billing_form' => __( 'Before Billing Form', Email_Marketing_Plugin::get_textdomain() ),
			'woocommerce_after_checkout_billing_form' => __( 'After Billing Form', Email_Marketing_Plugin::get_textdomain() ),
			'woocommerce_before_checkout_shipping_form' => __( 'Before Shipping Form', Email_Marketing_Plugin::get_textdomain() ),
			'woocommerce_after_checkout_shipping_form' => __( 'After Shipping Form', Email_Marketing_Plugin::get_textdomain() ),
			'woocommerce_before_order_notes' => __( 'Before Order Notes', Email_Marketing_Plugin::get_textdomain() ),
			'woocommerce_after_order_notes' => __( 'After Order Notes', Email_Marketing_Plugin::get_textdomain() )
		) );
	}

	/**
	 * Checks if WooCommerce is installed and activated by looking at the 'active_plugins' array
	 * @return bool True if WooCommerce is installed
	 */
	public static function is_woocommerce_installed() {
		require_once( ABSPATH . 'wp-admin/includes/plugin.php' );

		$slug = 'woocommerce/woocommerce.php';

		return is_plugin_active( $slug ) || is_plugin_active_for_network( $slug );
	}

	public static function init_wc_session() {
		if ( !WC()->session ) {
			include_once( WC()->plugin_path() .'/includes/abstracts/abstract-wc-session.php' );
			include_once( WC()->plugin_path() .'/includes/class-wc-session-handler.php' );
			WC()->session = new WC_Session_Handler();
		}
	}

	/**
	 * If the customer checked the checkbox displayed by self::render_checkout_optin(...),
	 * then add their billing email to the specified lists
	 *
	 * @param int $order_id
	 * @param array $posted
	 */
	public function maybe_subscribe_customer( $order_id, $posted_data ) {
		if ( isset( $_POST[Email_Marketing_Plugin::get_branding() . '_checkout_optin'] ) && $_POST[Email_Marketing_Plugin::get_branding() . '_checkout_optin'] === '1' ) {
			$this->plugin->get_api()->add_contact_to_lists( $posted_data['billing_email'], explode( ',', get_option( Email_Marketing_Plugin::get_branding() . '_wc_checkout_optin_lists' ) ), isset( $posted_data['billing_first_name'] ) ? $posted_data['billing_first_name'] : '', isset( $posted_data['billing_last_name'] ) ? $posted_data['billing_last_name'] : '' );
		}
	}
}
